@@ -1,9 +1,9 @@
-static char RcsId[] = "$Header: dbm_update.cpp,v 6.9 2000/03/15 17:07:57 taurel Rel $";
-
-/* $Log:	dbm_update.cpp,v $
-Revision 6.9  2000/03/15  17:07:57  17:07:57  taurel (E.Taurel)
-Change compilation variable from -solaris to sun
+static char RcsId[] = "$Header: /segfs/dbase/res/svc/ndbm/RCS/dbm_update.cpp,v 6.9 2000/03/15 17:07:57 taurel Rel goetz $";
 
+/* $Log: dbm_update.cpp,v $
+ * Revision 6.9  2000/03/15 17:07:57  taurel
+ * Change compilation variable from -solaris to sun
+ *
 Revision 6.8  2000/02/15  12:40:31  12:40:31  taurel (E.Taurel)
 Fix bug in update of arrays in the updres_1_svc function (file dbm_update.cpp). Change the value of the array flag when calling the upd_res function.
 
