@@ -1,9 +1,9 @@
-static char RcsId[] = "$Header: main_svc.cpp,v 6.9 2000/03/15 17:07:48 taurel Rel $";
-
-/* $Log:	main_svc.cpp,v $
-Revision 6.9  2000/03/15  17:07:48  17:07:48  taurel (E.Taurel)
-Change compilation variable from -solaris to sun
+static char RcsId[] = "$Header: /segfs/dbase/res/svc/ndbm/RCS/main_svc.cpp,v 6.9 2000/03/15 17:07:48 taurel Rel goetz $";
 
+/* $Log: main_svc.cpp,v $
+ * Revision 6.9  2000/03/15 17:07:48  taurel
+ * Change compilation variable from -solaris to sun
+ *
 Revision 6.8  2000/02/15  12:40:22  12:40:22  taurel (E.Taurel)
 Fix bug in update of arrays in the updres_1_svc function (file dbm_update.cpp). Change the value of the array flag when calling the upd_res function.
 
@@ -103,7 +103,7 @@ extern "C"
 {
   u_long gettransient(const char *);
   void get_myaddress(struct sockaddr_in *);
-  int gethostname(char *,size_t);
+// int gethostname(char *,size_t);
 }
 
 typedef char * (* DbRpcLocalFunc)(...);
