@@ -1,5 +1,5 @@
 static char RcsId[] = 
-"@(#)$Header: gen_api.c,v 8.14 2000/06/22 14:50:15 goetz Rel $";
+"@(#)$Header: gen_api.c,v 8.18 2000/11/12 13:24:01 goetz Rel $";
 /*+*******************************************************************
 
  File       :	gen_api.c
@@ -13,13 +13,25 @@ static char RcsId[] =
 
  Original   :	January 1991
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:15 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:01 $
 
  $Author: goetz $
 
  $Log:	gen_api.c,v $
- * Revision 8.14  2000/06/22  14:50:15  14:50:15  goetz (Andrew GOETZ)
+ * Revision 8.18  2000/11/12  13:24:01  13:24:01  goetz (Andy Goetz)
+ * added dynamic error handling; improved TANGO error treatment
+ * 
+ Revision 8.17  1900/09/26 13:36:18  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+ * Revision 8.16  2000/09/25  14:50:12  14:50:12  goetz (Andy Goetz)
+ * tango_api implements TANGO_HOST, stateless import and reimport
+ * 
+ * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+ * changed from tango database to host mysql, added support for float and double arrays
+ * 
+ * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
  * fixed bug in dev_event_listen() which caused server to crash
  * 
  Revision 8.13  2000/06/02 21:56:33  goetz
@@ -64,141 +76,8 @@ static char RcsId[] =
  Revision 8.0  1999/12/28 14:18:21  goetz
  added TANGO support for TACO dev_xxx() calls via -DTANGO for C++
 
- Revision 7.10  1999/11/25 08:27:09  goetz
- replaced fprintf(stderr) with printf; made startup() C++ compatible
-
- Revision 7.9  1999/11/22 20:18:55  goetz
- removed const from gettransient() prototype
-
- Revision 7.8  1999/11/21 20:45:34  goetz
- included all M.Diehl's patches (major changes to gettransient() + main())
-
- Revision 7.7  1999/11/18 22:28:49  goetz
- fixed event multi-client bug, free client handle after event_listen, timeout in dev_synch()
-
- Revision 7.6  1999/11/01 19:05:14  goetz
- make local copy of user name returned by getpwuid() in sec_api.c
-
- Revision 7.5  1999/08/06 17:43:57  goetz
- removed usage of varargs for g++ on solaris and hpux
-
- Revision 7.4  1999/07/09 05:15:25  goetz
- added M.Diehl's patch to DevServerSig.c to exit() after calling unregister_server()
-
- Revision 7.3  1999/06/07 15:26:47  goetz
- fixed bug with multi-nethost reimport, device name stored with nethost
-
- Revision 7.2  1999/05/12 15:21:44  goetz
- changed dev_event_fire() to void; fixed bug in dev_event_unlisten()
-
- * Revision 7.1  99/05/11  15:59:33  15:59:33  goetz (Andy Goetz)
- * replace static declaration of event_client[] array by malloc()
- * 
- Revision 7.0  1999/04/26 07:30:28  goetz
- implemented user events (added event_api.c)
-
- Revision 6.15  1999/03/27 09:00:08  goetz
- asynch_cleanup() only checks for asynch clients; C++ returns output arguments if DS_WARNING
-
- * Revision 6.14  99/02/27  15:21:06  15:21:06  goetz (Andy Goetz)
- * fixed reimport bug; disabled reimport for rpc version 1
- * 
- Revision 6.13  1999/02/05 00:50:14  goetz
- fixed reimport socket+memory leak; ported to aCC on HPUX; suppressed rpc errors
-
- * Revision 6.12  99/01/18  21:38:50  21:38:50  goetz (Andy Goetz)
- * suppressed test for RPC_TIMEDOUT in svc_check()
- * 
- * Revision 6.11  99/01/14  23:14:39  23:14:39  goetz (Andy Goetz)
- * dbase timeout = 4 s; unregister ver. 1; svc_check() handles RPC_TIMEDOUT
- * 
- * Revision 6.10  98/12/22  10:29:32  10:29:32  taurel (E.Taurel)
- * Added support for Solaris and HP C++ native compiler. Remove K_R C support
- * 
- * Revision 6.9  98/12/15  14:04:49  14:04:49  goetz (Andy Goetz)
- * unregister RPC; use pgm no. 1 as flag; dev_pending() checks timeout; extern C prototype
- * 
- * Revision 6.8  98/11/20  14:35:24  14:35:24  goetz (Andy Goetz)
- * unregister API_VERSION (4) correctly now + check return value of svc_register()
- * 
- * Revision 6.7  98/11/20  10:34:49  10:34:49  goetz (Andy Goetz)
- * improved gettrans_ut(); multi-config_flags=0; removde static client_data; fixed linux select() bug; only register pgm # 0,4,5; portmap address == 111
- * 
- * Revision 6.6  98/09/24  09:25:01  09:25:01  goetz (Andy Goetz)
- * fixed stupid type which reintroduced asynchronous memory leak
- * 
- * Revision 6.5  98/08/26  16:20:25  16:20:25  goetz (Andy Goetz)
- * added dynamic NETHOST allocation, fixed bug in db_setup_multi, ported to Irix 6.4
- * 
- * Revision 6.4  98/04/29  10:21:14  10:21:14  goetz (Andy Goetz)
- * asynchronous - memory leak fixed, OS9 bug fixed, stale handle cleanup, pending=300
- * 
- * Revision 6.3  97/12/10  15:28:22  15:28:22  meyer (J.Meyer)
- * Changed select() timeout in rpc_check_host() to 200ms
- * 
- * Revision 6.2  97/12/01  12:59:05  12:59:05  goetz (Andy Goetz)
- * intermediate checkin before opening 5.14.1 branch
- * 
- * Revision 6.1  97/11/20  21:49:15  21:49:15  goetz (Andy Goetz)
- * main() continue even if cannot register asynch rpc
- * 
- * Revision 6.0  97/11/03  17:22:16  17:22:16  goetz (Andy Goetz)
- * merged synchronous and asynchronous branches
- * 
- Revision 5.15  1997/10/24 15:26:55  klotz
- ported to NT NT 4.0 + NMSDK 5.0 ; tested with TextTalkds ; new DevServerMain.c
-
- * Revision 5.14  97/09/25  17:36:32  17:36:32  goetz (Andy Goetz)
- * fixed
- * 
- Revision 5.13  1997/09/03 17:34:25  meyer
- Added casting for ucc++ in DevServerSig.c
-
- * Revision 5.12  97/09/03  15:54:01  15:54:01  meyer (J.Meyer)
- * Changed OS9 signal handling from intercept() to signal().
- * 
- * Revision 5.11  97/08/05  14:19:14  14:19:14  goetz (Andy Goetz)
- * ported to VxWork 5.2, pgm. no. based on CRC, -m suppressed, -ve errors bug fix
- * 
- * Revision 5.10  97/03/14  17:26:05  17:26:05  dserver ()
- * fixed bug with open file handles after free
- * 
- * Revision 5.8.1.4  97/03/14  17:24:39  17:24:39  goetz (Andy Goetz)
- * fixed bug with open file handles after free
- * 
- * Revision 5.9  97/03/13  11:17:06  11:17:06  klotz (W.D. Klotz)
- * first part of NT4.0 port
- * 
- * Revision 5.8  97/01/31  12:06:14  12:06:14  goetz (Andy Goetz)
- * dev_protocol(), Linux+security bug fixes, dev_import() ignores timeout on NULL procedure, svc_run for OS9
- * 
- * Revision 5.7  97/01/14  08:57:13  08:57:13  goetz (Andy Goetz)
- * added stateless to dev_free(),_raw(),_asyn(),_protocol(),_timeout()
- * 
- * Revision 5.6  97/01/12  18:33:33  18:33:33  goetz (Andy Goetz)
- * dev_import() now stateless i.e. import works even if server is not running
- * 
- * Revision 5.5  97/01/08  17:02:19  17:02:19  goetz (Andy Goetz)
- * Ported to Linux (2.0.9)
- * 
- * Revision 5.4  96/11/27  09:58:06  09:58:06  goetz (Andy Goetz)
- * removed rpc admin interface from DevServerMain.c; putenv() suppressed for _UCC
- * 
- * Revision 5.3  96/11/26  16:24:33  16:24:33  goetz (Andy Goetz)
- * fixed bug (i_nethost=0) in dev_import_local()
- * 
- * Revision 5.2  96/11/26  13:30:54  13:30:54  goetz (Andy Goetz)
- * OICDevice (C++ wrapper for OIC) added + fixed multi-nethost memory leaks
- * 
- * Revision 5.1  96/10/31  18:02:20  18:02:20  goetz (Andy Goetz)
- * added multi-nethost support
- * 
- * Revision 5.0  96/10/31  17:59:11  17:59:11  goetz (Andy Goetz)
- * added multi-nethost support
- * 
-
- Copyright (c) 1990 by European Synchrotron Radiation Facility, 
-                       Grenoble, France
+ Copyright (c) 1990-2000 by European Synchrotron Radiation Facility, 
+                            Grenoble, France
 
 ********************************************************************-*/
 
@@ -232,7 +111,7 @@ extern _DLLFunc long setup_config_multi 	(char *nethost, long *error);
 static void msg_write 		(_Int msg_type, char *msg_string);
 static void msg_send 		(_Int msg_type);
 static void msg_clear 		(_Int msg_type);
-static long get_error_string 	(long error, char *error_str);
+static char *get_error_string 	(long error);
 
 /*
  * UCC has not prototype for putenv() , define one ! 
@@ -243,6 +122,8 @@ static long get_error_string 	(long error, char *error_str);
 long putenv(char *);
 #endif /* _OSK */
 
+char *dev_error_string=NULL; /* global pointer to dynamic error string */
+char *dev_error_stack=NULL; /* global pointer to dynamic error stack */
 
 /****************************************
  *          Globals	                *
@@ -297,9 +178,6 @@ static _message_buffer 	message_buffer [NUMBER_OF_MSG_TYPES];
  *
  */
 
-/*nethost_info 	multi_nethost[MIN_NETHOST];*/
-/*long max_nethost = MIN_NETHOST;*/
-
 nethost_info 	*multi_nethost;
 long max_nethost = 0;
 
@@ -571,25 +449,21 @@ _DLLFunc void dev_printerror (DevShort mode,char *fmt, char *str)
 
 void _DLLFunc dev_printerror_no (DevShort mode, char *comment, long dev_errno)
 {
-	char error_string [512];
-	char err_string [256];
+	char *error_string=NULL;
 
 	/*
          * get error string from the resource database
          */
 
-	get_error_string ( dev_errno, err_string );
+	error_string = get_error_string ( dev_errno);
 
 	if (comment != NULL)
 	{
-		sprintf (error_string, "%s : %s", comment, err_string);
-	}
-	else
-	{
-		sprintf (error_string, "%s", err_string);
+		dev_printerror (mode,"%s",comment);
 	}
 
 	dev_printerror (mode,"%s",error_string);
+	if (error_string != NULL) free(error_string);
 }
 
 
@@ -608,26 +482,18 @@ void _DLLFunc dev_printerror_no (DevShort mode, char *comment, long dev_errno)
 
 _DLLFunc char * dev_error_str (long dev_errno)
 {
-	char		err_string[256];
 	char		*error_string = NULL;
-	int	 	len;
 
 	/*
 	 * get error string from the resource database
 	 */
 
-	get_error_string ( dev_errno, err_string );
+	error_string = get_error_string (dev_errno);
 
-	len = strlen(err_string) + 1;
-	error_string = (char *)malloc (len);
-	sprintf (error_string,"%s", err_string);
+	return(error_string);
 
-	return (error_string);
 }
 
-
-
-
 /*
  * Attention: OS9 version 2.4 can not handle variable argument
  * lists. It will still use the old tricky function!!
@@ -2059,11 +1925,12 @@ long setup_config_multi (char *nethost, long *error)
  Return(s)  :   DS_OK or DS_NOTOK or DS_WARNING
 ***********************************************************************-*/
 
-static long get_error_string (long error, char *error_str)
+static char *get_error_string (long error)
 {
 	char				res_path[LONG_NAME_SIZE];
 	char				res_name[SHORT_NAME_SIZE];
 	char				*ret_str = NULL;
+	char				*error_str = NULL;
 	db_resource 	res_tab;
 	long				db_error;
 	unsigned short 	error_number_mask = 0xfff;
@@ -2074,6 +1941,8 @@ static long get_error_string (long error, char *error_str)
 
 	time_t		sec;
 	char		*time_stamp;
+	static char	*no_error_mess="No error message stored in the database for negative errors\n";
+	static char	*db_error_mess="Failed to get error message from database\n";
 
 
 #ifdef EBUG
@@ -2096,10 +1965,10 @@ static long get_error_string (long error, char *error_str)
  */
         if ( error < 0 )
         {
-                sprintf ( error_str,
-                          "%s  No error message stored in the database for negative errors (%d)",
-                          time_stamp, error );
-                return (DS_WARNING);
+		error_str = (char*)malloc(strlen(time_stamp)+strlen(no_error_mess)+2);
+                sprintf ( error_str, "%s %s", time_stamp, no_error_mess );
+                /*return (DS_WARNING);*/
+                return (error_str);
         }
 
 /*
@@ -2124,12 +1993,28 @@ static long get_error_string (long error, char *error_str)
 		{
 			if ((_Int)(DevErr_List[i].dev_errno) == (_Int)error)
 			{
-				sprintf ( error_str, "%s  %s", time_stamp, 
+				error_str = (char*)malloc(strlen(time_stamp)+
+					strlen(DevErr_List[i].message)+2);
+				sprintf ( error_str, "%s %s", time_stamp, 
 				    DevErr_List[i].message );
-				return (DS_OK);
+				/*return (DS_OK);*/
+				return (error_str);
 			}
 		}
 	}
+/*
+ * Treat dynamic errors by returning the dynamically generated server 
+ * error string rather than the static error string stored in the global
+ * error table or the database
+ */
+	if (dev_error_string != NULL)
+	{
+		error_str = (char*)malloc(strlen(time_stamp)+strlen(dev_error_string)+2);
+		sprintf ( error_str, "%s %s", time_stamp, dev_error_string );
+		error_str[strlen(error_str)] = 0;
+		/*return (DS_OK);*/
+		return (error_str);
+	}
 
 /*
  * Create the resource path and the resource structure.
@@ -2160,44 +2045,81 @@ static long get_error_string (long error, char *error_str)
 		    db_error);
 #endif /* EBUG */
 
-	   /*
- 	    * If the database call returned an error, search the global
-	    * error table for a description.
-	    */
+/*
+ * If the database call returned an error, search the global
+ * error table for a description.
+ */
 
 		for (i=0; i<(MAX_DEVERR); i++)
 		{
 			if ((_Int)(DevErr_List[i].dev_errno) == (_Int)db_error)
 			{
-				sprintf ( error_str, "%s  %s", time_stamp, 
+				error_str = (char*)malloc(strlen(time_stamp)+
+					strlen(DevErr_List[i].message)+2);
+				sprintf ( error_str, "%s %s", time_stamp, 
 				    DevErr_List[i].message );
-				return (DS_NOTOK);
+				/*return (DS_NOTOK);*/
+				return (error_str);
 			}
 		}
 
-		sprintf ( error_str, "%s  db_getresource() failed with error (%d)", 
-		    time_stamp, db_error );
-		return (DS_NOTOK);
+		error_str = (char*)malloc(strlen(time_stamp)+ strlen(db_error_mess)+2);
+		sprintf ( error_str, "%s %s", time_stamp, db_error_mess );
+		/*return (DS_NOTOK);*/
+		return (error_str);
 	}
 
-	/*
+/*
  * If the variable ret_str is still NULL, no resource value was found
  * in the database, but the function was executed without error.
  * In this case return the value DS_WARNING.
  */
 	if ( ret_str == NULL )
 	{
-		sprintf ( error_str, 
-		    "%s  No error message stored in the database for error (%d/%d/%d)", 
-		    time_stamp, team, server, error_ident );
-		return (DS_WARNING);
+		error_str = (char*)malloc(strlen(time_stamp)+strlen(no_error_mess)+2);
+                sprintf ( error_str, "%s %s", time_stamp, no_error_mess );
+		/*sprintf ( error_str, 
+		    "%s No error message stored in the database for error (%d/%d/%d)", 
+		    time_stamp, team, server, error_ident );*/
+		return (error_str);
 	}
 
-	/*
+/*
  * Found the error string in the database. Return the string with 
  * a time stamp.
  */
-	sprintf ( error_str, "%s  %s", time_stamp, ret_str );
+	error_str = (char*)malloc(strlen(time_stamp)+strlen(ret_str)+2);
+	sprintf ( error_str, "%s %s", time_stamp, ret_str );
 	free (ret_str);
-	return (DS_OK);
+	return (error_str);
+}
+
+/*+**********************************************************************
+ Function   :	long dev_error_push()
+
+ Description:   Push a dynamically generated error string onto the error 
+		string stack so that it can be transferred back to the client 
+		at the end of execution.
+
+ Arg(s) In  :   char *error_string    - error string
+
+ Arg(s) Out :   none
+
+ Return(s)  :   DS_OK or DS_NOTOK or DS_WARNING
+***********************************************************************-*/
+long dev_error_push (char *error_string)
+{
+	if (dev_error_stack == NULL)
+	{
+		dev_error_stack = (char*)malloc(strlen(error_string)+1);
+		dev_error_stack[0] = 0;
+	}
+	else
+	{
+		dev_error_stack = (char*)realloc(dev_error_stack, strlen(dev_error_stack)+strlen(error_string)+1);
+	}
+	sprintf(dev_error_stack+strlen(dev_error_stack),"%s",error_string);
+	dev_error_stack[strlen(dev_error_stack)] = 0;
+
+	return(DS_OK);
 }
