@@ -1,4 +1,4 @@
-static char RcsId[] = "@(#)$Header: tango_api.cpp,v 8.14 2000/06/22 14:50:24 goetz Rel $";
+static char RcsId[] = "@(#)$Header: tango_api.cpp,v 8.18 2000/11/12 13:24:01 goetz Rel $";
 /*+*******************************************************************
 
  File       :	tango_api.c
@@ -17,24 +17,36 @@ static char RcsId[] = "@(#)$Header: tango_api.cpp,v 8.14 2000/06/22 14:50:24 goe
 
  Original   :	December 1999
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:24 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:01 $
 
  $Author: goetz $
 
  $Log:	tango_api.cpp,v $
-// Revision 8.14  2000/06/22  14:50:24  14:50:24  goetz (Andrew GOETZ)
+// Revision 8.18  2000/11/12  13:24:01  13:24:01  goetz (Andy Goetz)
+// added dynamic error handling; improved TANGO error treatment
+// 
+ Revision 8.17  1900/09/26 13:36:27  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+// Revision 8.16  2000/09/25  14:50:12  14:50:12  goetz (Andy Goetz)
+// tango_api implements TANGO_HOST, stateless import and reimport
+// 
+// Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+// changed from tango database to host mysql, added support for float and double arrays
+// 
+// Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
 // fixed bug in dev_event_listen() which caused server to crash
 // 
- Revision 8.13  2000/06/02 21:56:42  goetz
- _DEVICE_H now _TANGO_DEVICE_H in Device.H; (char*)taco_tango
-
- Revision 8.12  2000/06/02 15:58:01  goetz
- dev_event_fire() now device specific; ported to SuSE V6.4
-
- Revision 8.11  2000/05/31 07:47:47  goetz
- tango_api has local copies of argc and argv, ported to HP-UX
-
+// Revision 8.13  2000/06/02 21:56:42  goetz
+// _DEVICE_H now _TANGO_DEVICE_H in Device.H; (char*)taco_tango
+//
+// Revision 8.12  2000/06/02 15:58:01  goetz
+// dev_event_fire() now device specific; ported to SuSE V6.4
+//
+// Revision 8.11  2000/05/31 07:47:47  goetz
+// tango_api has local copies of argc and argv, ported to HP-UX
+//
 // Revision 8.10  2000/05/30  10:16:33  10:16:33  goetz (Andrew GOETZ)
 // tango_api.cpp now has same version as other DSAPI files (V8.10)
 // 
@@ -68,7 +80,8 @@ static long tango_to_taco_type(long );
 static CORBA_Any tango_argin_to_any(long, long, void *, long *);    
 static void tango_any_to_argout(long, long, CORBA_Any, void *, long *);    
 static void tango_any_to_argout_raw(long, long, CORBA_Any, void *, long *);    
-
+static long tango_dev_check(devserver, long *);    
+static long tango_dev_error_string(Tango_DevFailed tango_exception);
 
 /*
  * TANGO/CORBA globals with file scope
@@ -86,6 +99,18 @@ typedef struct _tango_device {
 	Tango_Device_var object;} _tango_device;
 static vector<_tango_device> tango_device;
 static long n_tango_device=0;
+static string tango_host;
+static int tango_port=10000;
+
+/*
+ * symbols used by TACO/TANGO interface
+ */
+#define TANGO_DEV_NOT_IMPORTED 	0
+#define TANGO_DEV_IMPORTED 	1
+#define TANGO_BAD_CONNECTION	2
+
+extern long debug_flag;
+extern char *dev_error_string;
 
 /*+**********************************************************************
  Function   :   extern long tango_db_import()
@@ -104,8 +129,6 @@ static long n_tango_device=0;
 long tango_db_import(char *db_name, char *db_host, long db_port, long *error)
 {
 
-	cout4 << "tango_db_import(): called name = " << db_name << ", host = " << db_host << ", port = " << db_port << endl; 
-
 	if (db_name == NULL) {
 		*error = DevErr_DbImportFailed;
 		return(-1);
@@ -131,8 +154,10 @@ long tango_db_import(char *db_name, char *db_host, long db_port, long *error)
 		obj = orb->get_inet_object(db_host,db_port,db_name);
 		tango_dbase = Tango_Device::_narrow(obj);
 	}
-	catch (CORBA_Exception  &e) {
-		TangoUtil::print_exception(e); 
+	catch (CORBA_Exception  &corba_exception) {
+#ifdef DEBUG
+		if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception); 
+#endif
 		*error = DevErr_DbImportFailed;
 		return(DS_NOTOK);
 	}
@@ -159,17 +184,38 @@ long tango_dev_import(char *dev_name, long access, devserver *ds_ptr, long *erro
 
 	_tango_device tango_device_tmp;
 
-	cout4 << "tango_dev_import(): called name = " << dev_name  << endl; 
-
 	if (!tango_dbase_init) 
 	{
 		try
 		{
-			tango_db_import("database","amber1",10000,error);
+			char *tango_host_env_c_str;
+			char tango_host_c_str[256];
+
+			tango_host_env_c_str = getenv("TANGO_HOST");
+			if (tango_host_env_c_str == NULL)
+			{
+				cout <<"TANGO_HOST environment variable not set, set it and retry (e.g. TANGO_HOST=tango:10000)" << endl;
+				return(DS_NOTOK);
+			}
+			string tango_host_env(tango_host_env_c_str);
+			string::size_type separator;
+			
+			separator = tango_host_env.find(":");
+			if (separator != string::npos)
+			{
+				tango_port = atoi((tango_host_env.substr(separator+1)).c_str());
+			}
+			tango_host = tango_host_env.substr(0,separator);
+			
+			sprintf(tango_host_c_str,"%s",tango_host.c_str());
+			tango_db_import("database",tango_host_c_str,
+					 tango_port,error);
         	}
-		catch (CORBA_Exception  &e)
+		catch (CORBA_Exception  &corba_exception)
 		{
-                	TangoUtil::print_exception(e);
+#ifdef DEBUG
+                	if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
 			*error = DevErr_DbImportFailed;
                 	return(DS_NOTOK);
         	}
@@ -181,7 +227,7 @@ long tango_dev_import(char *dev_name, long access, devserver *ds_ptr, long *erro
 		return(DS_NOTOK);
 	}
 
-// Connect to TANGO database
+// Get IOR from TANGO database
 
 	try 
 	{
@@ -192,34 +238,45 @@ long tango_dev_import(char *dev_name, long access, devserver *ds_ptr, long *erro
 
                 Tango_DevVarLongStringArray *dev_info;
                 received >>= dev_info;
-
-		cout4 << "Device name : " << (dev_info->svalue)[0] << endl;
-		cout4 << "Device IOR : " << (dev_info->svalue)[1] << endl;
-
+//
+// device defined but not exported - case for stateless import
+//
                 if ((dev_info->lvalue)[0] == 0)
                 {
-			*error = DevErr_DeviceNotImportedYet;
-                        return(DS_NOTOK);
+			if (dev_notimported_init(dev_name,access,0,ds_ptr,error) == DS_NOTOK)
+				return(DS_NOTOK);
+			(*ds_ptr)->ds_id = -1;
+			(*ds_ptr)->dev_access = access;
+			(*ds_ptr)->rpc_protocol = D_IIOP;
+                        return(DS_OK);
                 }
                 CORBA_Object_var obj = orb->string_to_object((dev_info->svalue)[1]
 );
 
                 tango_device_tmp.object = Tango_Device::_narrow(obj);
-                tango_device_tmp.flag = 1;
+                tango_device_tmp.flag = TANGO_DEV_IMPORTED;
         }
-        catch (CORBA_Exception  &e)
+//
+// failed to import - device does not exist ?
+//
+        catch (CORBA_Exception  &corba_exception)
 	{
-                TangoUtil::print_exception(e);
-		*error = DevErr_DeviceNotImportedYet;
+#ifdef DEBUG
+                if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
+		*error = DevErr_DeviceOfThisNameNotServed;
                 return(DS_NOTOK);
         }
-
+//
+// device defined but could not be narrowed - case for stateless import
+//
         if (CORBA_is_nil(tango_device_tmp.object))
         {
-		*error = DevErr_DeviceNotImportedYet;
-                return(DS_NOTOK);
+		if (dev_notimported_init(dev_name,access,0,ds_ptr,error) == DS_NOTOK)                                return(DS_NOTOK);
+                (*ds_ptr)->ds_id = -1;
+                (*ds_ptr)->rpc_protocol = D_IIOP;
+                return(DS_OK);
         }
-        cout4 << "Connected to device" << endl;
 //
 // initialise command list for device
 //
@@ -228,9 +285,7 @@ long tango_dev_import(char *dev_name, long access, devserver *ds_ptr, long *erro
 		Tango_DevCmdInfoList_var cmd_query = tango_device_tmp.object->command_list_query();
 		Tango_DevInfo_var dev_info = tango_device_tmp.object->info();
 
-		cout4 << "Number of command(s) : " << cmd_query->length() << endl;
 		tango_device_tmp.cmd_name = vector<string>(cmd_query->length());
-// Q: why can't I create a vector with an initialising value ?
 		tango_device_tmp.cmd_value = vector<long>(cmd_query->length());
 		tango_device_tmp.argin_type = vector<long>(cmd_query->length());
 		tango_device_tmp.argout_type = vector<long>(cmd_query->length());
@@ -238,16 +293,12 @@ long tango_dev_import(char *dev_name, long access, devserver *ds_ptr, long *erro
 		{
 			long error;
 			tango_device_tmp.cmd_name[i] = string((*cmd_query)[i].cmd_name);
-			cout4 << "Command name : " << tango_device_tmp.cmd_name[i].c_str();
 			get_cmd_value((char*)dev_info->dev_class,
 			              (char*)tango_device_tmp.cmd_name[i].c_str(),
 			              &tango_device_tmp.cmd_value[i],
 			              &error);
-			cout4 << "value " << tango_device_tmp.cmd_value[i];
 			tango_device_tmp.argin_type[i] = (*cmd_query)[i].in_type;
-			cout4 << " ->Input type : " << Tango_CmdArgTypeName[tango_device_tmp.argin_type[i]];
 			tango_device_tmp.argout_type[i] = (*cmd_query)[i].out_type;
-			cout4 << " ->Output type : " << Tango_CmdArgTypeName[tango_device_tmp.argout_type[i]] << endl;
 		}
         	if ((*ds_ptr = (devserver)malloc(sizeof(struct _devserver))) == NULL)
         	{
@@ -271,17 +322,100 @@ long tango_dev_import(char *dev_name, long access, devserver *ds_ptr, long *erro
         	(*ds_ptr)->rpc_timeout.tv_usec = 0;
 
 	}
-        catch (CORBA_Exception  &e)
+        catch (CORBA_Exception  &corba_exception)
         {
-                TangoUtil::print_exception(e);
-		*error = DevErr_DeviceNotImportedYet;
-                return(DS_NOTOK);
+#ifdef DEBUG
+                if (debug_flag & DBG_TANGO) TangoUtil::print_exception(e);
+#endif
+//
+// device command query failed - case for stateless import
+// 
+                if (dev_notimported_init(dev_name,access,0,ds_ptr,error) == DS_NOTOK)                                return(DS_NOTOK);
+                (*ds_ptr)->ds_id = -1;
+                (*ds_ptr)->rpc_protocol = D_IIOP; 
+                return(DS_OK);
         }
 	n_tango_device++;
 
 	return(0);
 }
 
+/*+**********************************************************************
+ Function   :   extern long tango_dev_reimport()
+
+ Description:   reimport TANGO device
+
+ Arg(s) In  :   char *dev_name - name of device
+
+ Arg(s) Out :   none
+
+ Return(s)  :   DS_OK or DS_NOTOK
+***********************************************************************-*/
+
+long tango_dev_reimport(devserver ds, long *error)
+{
+
+	if (!tango_dbase_init) 
+	{
+		try
+		{
+			char tango_host_c_str[256];
+			sprintf(tango_host_c_str,"%s",tango_host.c_str());
+
+			tango_db_import("database",tango_host_c_str,
+					 tango_port,error);
+        	}
+		catch (CORBA_Exception  &corba_exception)
+		{
+#ifdef DEBUG
+                	if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
+			*error = DevErr_DbImportFailed;
+                	return(DS_NOTOK);
+        	}
+	}
+
+// Get new IOR from TANGO database (server might have been restarted)
+
+	try 
+	{
+                CORBA_Any send;
+                send <<= ds->device_name;
+
+                CORBA_Any_var received = tango_dbase->command_inout("DbImportDevice",send);
+
+                Tango_DevVarLongStringArray *dev_info;
+                received >>= dev_info;
+
+                if ((dev_info->lvalue)[0] == 0)
+                {
+			*error = DevErr_BadServerConnection;
+                        return(DS_NOTOK);
+                }
+                CORBA_Object_var obj = orb->string_to_object((dev_info->svalue)[1]
+);
+
+                tango_device[ds->ds_id].object = Tango_Device::_narrow(obj);
+                tango_device[ds->ds_id].flag = TANGO_DEV_IMPORTED;
+        }
+        catch (CORBA_Exception  &corba_exception)
+	{
+#ifdef DEBUG
+                if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
+		*error = DevErr_BadServerConnection;
+                return(DS_NOTOK);
+        }
+
+        if (CORBA_is_nil(tango_device[ds->ds_id].object))
+        {
+		*error = DevErr_BadServerConnection;
+                return(DS_NOTOK);
+        }
+
+	return(0);
+}
+
 /*+**********************************************************************
  Function   :   extern long tango_dev_putget()
 
@@ -306,21 +440,17 @@ long tango_dev_putget(devserver ds, long cmd, void *argin, long argin_type,
 	long i_cmd = -1, dev_id;
 	char *cmd_name = NULL;
 
-	cout4 << "tango_dev_putget(): called device id = " << dev_id  << " command = " << cmd << endl; 
+	if (tango_dev_check(ds,error) != DS_OK) return(DS_NOTOK);
 
 	dev_id = ds->ds_id;
 //
 // check if command has already been executed, if not then
 // read command string using its value from TACO database
 //
-// Q: why can't I use find() in a vector to return the index of the element ?
 	for (long i=0; i<tango_device[dev_id].cmd_value.size(); i++)
 	{
 		if (tango_device[dev_id].cmd_value[i] == cmd)
 		{
-			cout4 << "tango_dev_putget(): found command at position "<< i;
-			cout4 << " argin_type " << tango_device[dev_id].argin_type[i] ;
-			cout4 << " argout_type " << tango_device[dev_id].argout_type[i] << endl;
 			i_cmd = i;
 			break;
 		}
@@ -331,29 +461,96 @@ long tango_dev_putget(devserver ds, long cmd, void *argin, long argin_type,
 		return(DS_NOTOK);
 	}
 
-	try 
+	if (tango_device[dev_id].flag == TANGO_DEV_IMPORTED) 
 	{
+		try 
+		{
 
 // insert input argument into Any type
 
-		CORBA_Any send = tango_argin_to_any(argin_type, tango_device[dev_id].argin_type[i_cmd],
-		                                    argin, error);
+			CORBA_Any send = tango_argin_to_any(argin_type, tango_device[dev_id].argin_type[i_cmd],
+			                                    argin, error);
 
 // execute command using command_inout() method
 
-                CORBA_Any_var received = tango_device[dev_id].object->command_inout(tango_device[dev_id].cmd_name[i_cmd].c_str(),send);
+                	CORBA_Any_var received = tango_device[dev_id].object->command_inout(tango_device[dev_id].cmd_name[i_cmd].c_str(),send);
 
 // extract output argument from Any
 
-		tango_any_to_argout(argout_type, tango_device[dev_id].argout_type[i_cmd],
-		                    received, argout, error);
-        }
-        catch (CORBA_Exception  &e)                                                     {
-                TangoUtil::print_exception(e);
-// TODO - improve TANGO to TACO error handling here
-		*error = DevErr_CommandFailed;
-                return(DS_NOTOK);
-        }
+			tango_any_to_argout(argout_type, tango_device[dev_id].argout_type[i_cmd],
+		                    	received, argout, error);
+        	}
+        	catch (Tango_DevFailed  &tango_exception)
+		{
+#ifdef DEBUG
+                	if (debug_flag & DBG_TANGO) TangoUtil::print_exception(tango_exception);
+#endif
+//
+// recover TANGO error string and save it in global error string so that
+// it can be printed out with dev_error_str()
+//
+			tango_dev_error_string(tango_exception);
+			*error = DevErr_CommandFailed;
+                	return(DS_NOTOK);
+        	}
+//
+// CORBA exception, assume server is down, try to reimport immediately
+//
+                catch (CORBA_Exception  &e)
+                {
+#ifdef DEBUG
+                        if (debug_flag & DBG_TANGO) TangoUtil::print_exception(e);
+#endif
+                        tango_device[dev_id].flag = TANGO_BAD_CONNECTION;
+			if (tango_dev_check(ds,error) != DS_OK)
+			{
+		        	*error = DevErr_BadServerConnection;
+                        	return(DS_NOTOK);
+			}
+//
+// if import works then execute the command again i.e. immediate reconnection
+
+			try 
+			{
+		
+		// insert input argument into Any type
+		
+				CORBA_Any send = tango_argin_to_any(argin_type, tango_device[dev_id].argin_type[i_cmd],
+			                                    	argin, error);
+		
+// execute command using command_inout() method
+		
+		               	CORBA_Any_var received = tango_device[dev_id].object->command_inout(tango_device[dev_id].cmd_name[i_cmd].c_str(),send);
+		
+// extract output argument from Any
+		
+				tango_any_to_argout(argout_type, tango_device[dev_id].argout_type[i_cmd],
+			                    	received, argout, error);
+		        }
+		        catch (Tango_DevFailed  &tango_exception)
+		        {
+#ifdef DEBUG
+		        	if (debug_flag & DBG_TANGO) TangoUtil::print_exception(tango_exception);
+#endif
+//
+// recover TANGO error string and save it in global error string so that
+// it can be printed out with dev_error_str()
+//
+				tango_dev_error_string(tango_exception);
+	                        *error = DevErr_CommandFailed;
+	                        return(DS_NOTOK);
+	                }
+	                catch (CORBA_Exception  &corba_exception)
+	                {
+#ifdef DEBUG
+	                        if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
+	                        tango_device[dev_id].flag = TANGO_BAD_CONNECTION;
+	                        *error = DevErr_BadServerConnection;
+	                        return(DS_NOTOK);
+	                }
+		}
+	}
 
 	return(0);
 }
@@ -383,21 +580,17 @@ long tango_dev_putget_raw(devserver ds, long cmd, void *argin, long argin_type,
 	long i_cmd = -1, dev_id;
 	char *cmd_name = NULL;
 
-	cout4 << "tango_dev_putget(): called device id = " << dev_id  << " command = " << cmd << endl; 
+	if (tango_dev_check(ds,error) != DS_OK) return(DS_NOTOK);
 
 	dev_id = ds->ds_id;
 //
 // check if command has already been executed, if not then
 // read command string using its value from TACO database
 //
-// Q: why can't I use find() in a vector to return the index of the element ?
 	for (long i=0; i<tango_device[dev_id].cmd_value.size(); i++)
 	{
 		if (tango_device[dev_id].cmd_value[i] == cmd)
 		{
-			cout4 << "tango_dev_putget(): found command at position "<< i;
-			cout4 << " argin_type " << tango_device[dev_id].argin_type[i] ;
-			cout4 << " argout_type " << tango_device[dev_id].argout_type[i] << endl;
 			i_cmd = i;
 			break;
 		}
@@ -408,31 +601,90 @@ long tango_dev_putget_raw(devserver ds, long cmd, void *argin, long argin_type,
 		return(DS_NOTOK);
 	}
 
-	try 
-	{
+        if (tango_device[dev_id].flag == TANGO_DEV_IMPORTED)
+        {
+		try 
+		{
 
 // insert input argument into Any type
 
-		CORBA_Any send = tango_argin_to_any(argin_type, tango_device[dev_id].argin_type[i_cmd],
-		                                    argin, error);
+			CORBA_Any send = tango_argin_to_any(argin_type, tango_device[dev_id].argin_type[i_cmd],
+		                                    	argin, error);
 
 // execute command using command_inout() method
 
-                CORBA_Any_var received = tango_device[dev_id].object->command_inout(tango_device[dev_id].cmd_name[i_cmd].c_str(),send);
+                	CORBA_Any_var received = tango_device[dev_id].object->command_inout(tango_device[dev_id].cmd_name[i_cmd].c_str(),send);
 
 // extract output argument from Any
 
-		tango_any_to_argout_raw(argout_type, tango_device[dev_id].argout_type[i_cmd],
-		                    received, argout, error);
-        }
-        catch (CORBA_Exception  &e)                                                     {
-                TangoUtil::print_exception(e);
+			tango_any_to_argout_raw(argout_type, tango_device[dev_id].argout_type[i_cmd],
+		                    	received, argout, error);
+        	}
+                catch (Tango_DevFailed  &tango_exception)
+                {
+#ifdef DEBUG
+                        if (debug_flag & DBG_TANGO) TangoUtil::print_exception(tango_exception);
+#endif
 // TODO - improve TANGO to TACO error handling here
-		*error = DevErr_CommandFailed;
-                return(DS_NOTOK);
-        }
+                        *error = DevErr_CommandFailed;
+                        return(DS_NOTOK);
+                }
+//
+// CORBA exception, assume server is down, try to reimport immediately
+//
+                catch (CORBA_Exception  &corba_exception)
+                {
+#ifdef DEBUG
+                        if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
+                        tango_device[dev_id].flag = TANGO_BAD_CONNECTION;
+			if (tango_dev_check(ds,error) != DS_OK)
+			{
+		        	*error = DevErr_BadServerConnection;
+                        	return(DS_NOTOK);
+			}
+//
+// if import works then execute the command again i.e. immediate reconnection
 
-	return(0);
+			try 
+			{
+		
+		// insert input argument into Any type
+		
+				CORBA_Any send = tango_argin_to_any(argin_type, tango_device[dev_id].argin_type[i_cmd],
+			                                    	argin, error);
+		
+// execute command using command_inout() method
+		
+		               	CORBA_Any_var received = tango_device[dev_id].object->command_inout(tango_device[dev_id].cmd_name[i_cmd].c_str(),send);
+		
+// extract output argument from Any
+		
+				tango_any_to_argout_raw(argout_type, tango_device[dev_id].argout_type[i_cmd],
+			                    	received, argout, error);
+		        }
+		        catch (Tango_DevFailed  &tango_exception)
+		        {
+#ifdef DEBUG
+		        	if (debug_flag & DBG_TANGO) TangoUtil::print_exception(tango_exception);
+#endif
+// TODO - improve TANGO to TACO error handling here
+	                        *error = DevErr_CommandFailed;
+	                        return(DS_NOTOK);
+	                }
+	                catch (CORBA_Exception  &corba_exception)
+	                {
+#ifdef DEBUG
+	                        if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
+	                        tango_device[dev_id].flag = TANGO_BAD_CONNECTION;
+	                        *error = DevErr_BadServerConnection;
+	                        return(DS_NOTOK);
+	                }
+		}
+	}
+
+	return(DS_OK);
 }
 
 
@@ -463,6 +715,8 @@ long tango_dev_cmd_query (devserver ds, DevVarCmdArray *varcmdarr, long *error)
 {
 	long dev_id;
 
+	if (tango_dev_check(ds,error) != DS_OK) return(DS_NOTOK);
+
 	dev_id = ds->ds_id;
 
 	*error = 0;
@@ -512,6 +766,8 @@ long tango_dev_ping (devserver ds, long *error)
 {
 	long dev_id;
 
+	if (tango_dev_check(ds,error) != DS_OK) return(DS_NOTOK);
+
 	dev_id = ds->ds_id;
 
 	*error = 0;
@@ -520,7 +776,11 @@ long tango_dev_ping (devserver ds, long *error)
         {
                 tango_device[dev_id].object->ping();
         }
-        catch (CORBA_Exception  &e)                                                     {                                                                                       TangoUtil::print_exception(e);
+        catch (CORBA_Exception  &corba_exception)
+	{
+#ifdef DEBUG
+		if (debug_flag & DBG_TANGO) TangoUtil::print_exception(corba_exception);
+#endif
 // TODO - improve TANGO to TACO error handling here
                 *error = DevErr_CommandFailed;
                 return(DS_NOTOK);
@@ -546,8 +806,6 @@ static CORBA_Any tango_argin_to_any(long argin_type, long tango_type, void *argi
 {
 	CORBA_Any send;
 
-	cout4 << "tango_argin_to_any(): convert argin_type " << argin_type << " to CORBA_Any" << endl;
-
 	if (argin == NULL && argin_type != D_VOID_TYPE)
 	{
 		throw_exception((const char*)"argin is NULL ", 
@@ -785,8 +1043,6 @@ static CORBA_Any tango_argin_to_any(long argin_type, long tango_type, void *argi
 
 static void tango_any_to_argout(long argout_type, long tango_type, CORBA_Any received, void *argout, long *error)
 {
-	cout4 << "tango_any_to_argout(): convert CORBA_Any to argout type " << argout_type << endl;
-
 	if (argout == NULL && argout_type != D_VOID_TYPE)
 	{
 		throw_exception((const char*)"argout is NULL ", 
@@ -898,7 +1154,6 @@ static void tango_any_to_argout(long argout_type, long tango_type, CORBA_Any rec
 				                Tango_APILIB);
 			}
 			received >>= tango_str;
-			cout4 << "tango_any_to_argout(): string received = " << tango_str << endl;
 			argout_str = (char**)argout;
 			*argout_str = (char*)malloc(strlen(tango_str)+1);
 			strcpy(*argout_str,tango_str);
@@ -918,7 +1173,6 @@ static void tango_any_to_argout(long argout_type, long tango_type, CORBA_Any rec
 				                Tango_APILIB);
 			}
 			received >>= tango_vsa;
-			cout4 << "tango_any_to_argout(): string array received length = " << tango_vsa->length() << endl;
 			argout_vsa = (DevVarStringArray*)argout;
 			if (argout_vsa->sequence == NULL)
 			{
@@ -1075,8 +1329,6 @@ static void tango_any_to_argout_raw(long argout_type, long tango_type, CORBA_Any
 
 	taco_opaque = (DevOpaque*)argout;
 
-	cout4 << "tango_any_to_argout_raw(): convert CORBA_Any to argout type " << argout_type << endl;
-
 	if (argout == NULL && argout_type != D_VOID_TYPE)
 	{
 		throw_exception((const char*)"argout is NULL ", 
@@ -1211,7 +1463,6 @@ static void tango_any_to_argout_raw(long argout_type, long tango_type, CORBA_Any
 				                Tango_APILIB);
 			}
 			received >>= tango_str;
-			cout4 << "tango_any_to_argout(): string received = " << tango_str << endl;
 			taco_str = (char*)malloc(strlen(tango_str)+1);
 			strcpy(taco_str,tango_str);
                         dc_datacmd.argout_type = argout_type;
@@ -1237,7 +1488,6 @@ static void tango_any_to_argout_raw(long argout_type, long tango_type, CORBA_Any
 				                Tango_APILIB);
 			}
 			received >>= tango_vsa;
-			cout4 << "tango_any_to_argout(): string array received length = " << tango_vsa->length() << endl;
 			taco_vsa.sequence = (char**)malloc(tango_vsa->length()*sizeof(char*));
 			taco_vsa.length = tango_vsa->length();
 			for (i=0; i< tango_vsa->length(); i++)
@@ -1445,6 +1695,12 @@ static long tango_to_taco_type(long tango_type)
 		case Tango_DEVVAR_ULONGARRAY : 
 			taco_type = D_VAR_ULONGARR;
 			break;
+		case Tango_DEVVAR_FLOATARRAY : 
+			taco_type = D_VAR_FLOATARR;
+			break;
+		case Tango_DEVVAR_DOUBLEARRAY : 
+			taco_type = D_VAR_DOUBLEARR;
+			break;
 		case Tango_DEVVAR_STRINGARRAY : 
 			taco_type = D_VAR_STRINGARR;
 			break;
@@ -1487,7 +1743,6 @@ static long get_cmd_value (char *class_name, char *cmd_name, long *cmd_value, lo
 
 	*error = 0;
 
-	cout4 << "Class " << class_name << " command " << cmd_name << endl;
 /*
  * Create the resource path and the resource structure.
  *
@@ -1529,3 +1784,105 @@ static long get_cmd_value (char *class_name, char *cmd_name, long *cmd_value, lo
 
 	return (DS_OK);
 }
+
+/*+**********************************************************************
+ Function   :   static long tango_dev_check()
+ 
+ Description:   Check TANGO device connection. If not imported then
+		import it. If bad connection has been signalled then reimport.	
+		Returns DS_OK if device has been (re)imported correctly.
+ 
+ Arg(s) In  :   devserver ds - client handle
+ 
+ Arg(s) Out :   devserver ds - updated if device (re)imported correctly
+ 
+ Return(s)  :   DS_OK or DS_NOTOK
+***********************************************************************-*/
+
+static long tango_dev_check(devserver ds, long *error)
+{
+	long dev_id;
+
+	dev_id = ds->ds_id;
+//
+// is device imported yet ?
+//
+        if (dev_id == -1)
+        {
+                devserver ds_ptr;
+                if ( tango_dev_import(ds->device_name, ds->dev_access, &ds_ptr, error)
+                     != DS_OK) return(DS_NOTOK);
+//
+// was device really imported ?
+//
+                if (ds_ptr->ds_id == -1)                                           
+                {
+                        free(ds_ptr);
+                        *error = DevErr_DeviceNotImportedYet;
+                        return(DS_NOTOK);
+                }
+//
+// device really imported - update device info in client's copy of devserver structure
+//
+                sprintf(ds->device_name,"%s",ds_ptr->device_name);
+                sprintf(ds->device_class,"%s",ds_ptr->device_class);
+                sprintf(ds->device_type,"%s",ds_ptr->device_type);
+                sprintf(ds->server_name,"%s",ds_ptr->server_name);
+                sprintf(ds->server_host,"%s",ds_ptr->server_host);
+                ds->clnt                = ds_ptr->clnt;
+                ds->ds_id               = ds_ptr->ds_id;
+                ds->no_svr_conn         = ds_ptr->no_svr_conn;
+                ds->rpc_conn_counter    = ds_ptr->rpc_conn_counter;
+                ds->dev_access          = ds_ptr->dev_access;
+                ds->i_nethost           = ds_ptr->i_nethost;
+                ds->rpc_protocol        = ds_ptr->rpc_protocol;
+                ds->rpc_timeout         = ds_ptr->rpc_timeout;
+                free(ds_ptr);
+        }
+        else
+        {
+//
+// does device need to be reimported ?
+//
+                if (tango_device[dev_id].flag == TANGO_BAD_CONNECTION)
+                {
+                        if (tango_dev_reimport(ds, error) != DS_OK)
+                        {
+                                *error = DevErr_BadServerConnection;               
+                                return(DS_NOTOK);                                  
+                        }                                                          
+                }                                                                  
+        }                                                                          
+
+	return(DS_OK);
+}
+/*+**********************************************************************
+ Function   :   static long tango_dev_error_string()
+ 
+ Description:   Recover TANGO error string stack from DevFailed exception.
+ 
+ Arg(s) In  :   Tango_DevFailed e - DevFailed exception
+ 
+ Arg(s) Out :   char *dev_error_string - global pointer to error string
+ 
+ Return(s)  :   DS_OK or DS_NOTOK
+***********************************************************************-*/
+
+static long tango_dev_error_string(Tango_DevFailed tango_exception)
+{
+	for (int i=0; i<tango_exception.errors.length(); i++)
+	{
+		if (i == 0)
+		{
+			dev_error_string = (char*)malloc(strlen(tango_exception.errors[i].reason.in())+1);
+			sprintf(dev_error_string,"%s",tango_exception.errors[i].reason.in());
+		}
+		else
+		{
+			dev_error_string = (char*)realloc(dev_error_string,strlen(dev_error_string)+strlen(tango_exception.errors[i].reason.in())+1);
+			sprintf(dev_error_string+strlen(dev_error_string),"%s",tango_exception.errors[i].reason.in());
+		}
+		printf("tango_dev_error_string(): %s",tango_exception.errors[i].reason.in());
+	}
+	return(DS_OK);
+}
